import { chainSupportsEIP1559 } from "./chainid.js";
import { parseUnits } from "@ethersproject/units";
export function makeGwei(n) { return parseUnits(n, "gwei"); }
/**
 * populateGasOptions adds gas limit, fee, and/or price options as provided in the `gasOptions` parameter to a built,
 * but as of yet unsent {@link PopulatedTransaction} object.
 *
 * @param {PopulatedTransaction} txn Ethers.js {@link PopulatedTransaction} object to add gas params to.
 * @param {GasOptions} gasOptions Gas limit/fee/price options for {@link PopulatedTransaction} objects.
 * @param {number} chainId Chain ID of the network which `txn` will be sent on, used for determining EIP-1559 support.
 * @param {boolean} ignoreLimit Optional. If set to `true`, the `gasLimit` field of `gasOptions` is entirely ignored.
 *
 * @return {PopulatedTransaction} Transaction object as passed as a param, but with any of `gasPrice`, `gasLimit`,
 * `maxFeePerGas`, and `maxPriorityFeePerGas` respectively set to values provided in `gasOptions`.
 */
export function populateGasOptions(txn, gasOptions, chainId, ignoreLimit) {
    if (ignoreLimit === void 0) { ignoreLimit = false; }
    if (chainSupportsEIP1559(chainId)) {
        if (gasOptions === null || gasOptions === void 0 ? void 0 : gasOptions.gasPrice) {
            txn.gasPrice = gasOptions.gasPrice;
        }
    }
    else {
        if (gasOptions === null || gasOptions === void 0 ? void 0 : gasOptions.maxFeePerGas) {
            txn.maxFeePerGas = gasOptions.maxFeePerGas;
        }
        if (gasOptions === null || gasOptions === void 0 ? void 0 : gasOptions.maxPriorityFeePerGas) {
            txn.maxPriorityFeePerGas = gasOptions.maxPriorityFeePerGas;
        }
    }
    if (!ignoreLimit) {
        if (gasOptions === null || gasOptions === void 0 ? void 0 : gasOptions.gasLimit) {
            txn.gasLimit = gasOptions.gasLimit;
        }
    }
    return txn;
}
export function makeTransactionGasOverrides(gasOptions, chainId, ignoreLimit) {
    if (ignoreLimit === void 0) { ignoreLimit = false; }
    var overrides = {};
    if (chainSupportsEIP1559(chainId)) {
        if (gasOptions === null || gasOptions === void 0 ? void 0 : gasOptions.gasPrice) {
            overrides.gasPrice = gasOptions.gasPrice;
        }
    }
    else {
        if (gasOptions === null || gasOptions === void 0 ? void 0 : gasOptions.maxFeePerGas) {
            overrides.maxFeePerGas = gasOptions.maxFeePerGas;
        }
        if (gasOptions === null || gasOptions === void 0 ? void 0 : gasOptions.maxPriorityFeePerGas) {
            overrides.maxPriorityFeePerGas = gasOptions.maxPriorityFeePerGas;
        }
    }
    if (!ignoreLimit) {
        if (gasOptions === null || gasOptions === void 0 ? void 0 : gasOptions.gasLimit) {
            overrides.gasLimit = gasOptions.gasLimit;
        }
    }
    return overrides;
}
